<?php
namespace App\DataTables;

use App\Models\CustomField;
use App\Models\Order;
use Barryvdh\DomPDF\Facade as PDF;
use Yajra\DataTables\EloquentDataTable;
use Yajra\DataTables\Services\DataTable;
use DB;
class OrderDataTable extends DataTable
{
    /**
     * custom fields columns
     * @var array
     */
    public static $customFields = [];

    /**
     * Build DataTable class.
     *
     * @param mixed $query Results from query() method.
     * @return \Yajra\DataTables\DataTableAbstract
     */
    public function dataTable($query)
    {
        
        $dataTable = new EloquentDataTable($query);
        $columns = array_column($this->getColumns(), 'data');
       
        $dataTable = $dataTable
            ->editColumn('id', function ($order) {
                return "#".$order->id;
            })
            
            ->editColumn('updated_at', function ($order) {
                return getDateColumn($order, 'updated_at');
            })
            ->editColumn('delivery_fee', function ($order) {
                return getPriceColumn($order, 'delivery_fee');
            })
            ->editColumn('tax', function ($order) {
                return $order->tax . "%";
            })
            ->editColumn('payment.status', function ($order) {
                return getPayment($order->payment,'status');
            })
            ->editColumn('active', function ($product) {
                return getBooleanColumn($product, 'active');
            })
            ->addColumn('action', 'orders.datatables_actions')
            ->rawColumns(array_merge($columns, ['action']));

        return $dataTable;
    }

    /**
     * Get columns.
     *
     * @return array
     */
    protected function getColumns()
    {
       
        $columns = [
            [
                'data' => 'increment_id',
                'title' => trans('lang.order_id'),
                'searchable' => true,

            ],
            [
                'data' => 'customer_firstname',
                'name' => 'user.name',
                'title' => trans('lang.order_user_id'),
                'searchable' => true,

            ],
            [
                // 'data' => 'status',
                'data' => 'orstatus',
                'name' => 'orderStatus.status',
                'title' => trans('lang.order_order_status_id'),

            ],
            [
                'data' => 'storename',
                'title' => "Store",
                'searchable' => false,

            ],
            [
                'data' => 'base_grand_total',
                'title' => "Grand Total",
                'searchable' => false,

            ],
               [
                'data' => 'outofstockqty',
                'name' => 'Out of Stock',
                'title' => "Out of Stock",

            ],
            [
                'data' => 'total_item_count',
                'name' => 'payment.status',
                'title' => "Total Qty",

            ],
            // [
            //     'data' => 'payment.method',
            //     'name' => 'payment.method',
            //     'title' => trans('lang.payment_method'),

            // ],
            // [
            //     'data' => 'active',
            //     'title' => trans('lang.order_active'),

            // ],
            [
                'data' => 'delivery_date1',
                'title' => 'Delivery Date and Slot Time',
                'searchable' => false,
                'orderable' => true,

            ],
            [
                'data' => 'updated_at',
                'title' => trans('lang.order_updated_at'),
                'searchable' => false,
                'orderable' => true,
            ],
        ];

        // $hasCustomField = in_array(Order::class, setting('custom_field_models', []));
         
        // if ($hasCustomField) {
        //     $customFieldsCollection = CustomField::where('custom_field_model', Order::class)->where('in_table', '=', true)->get();
          
        //     foreach ($customFieldsCollection as $key => $field) {
        //         array_splice($columns, $field->order - 1, 0, [[
        //             'data' => 'custom_fields.' . $field->name . '.view',
        //             'title' => trans('lang.order_' . $field->name),
        //             'orderable' => false,
        //             'searchable' => false,
        //         ]]);
        //     }
        // }
        
        return $columns;
    }

    /**
     * Get query source of dataTable.
     *
     * @param \App\Models\Post $model
     * @return \Illuminate\Database\Eloquent\Builder
     */
    public function query(Order $model)
    {
        if (auth()->user()->hasRole('admin')) {
           
   $domain_address_code = session('domain_address_setup'); 
   if(empty($domain_address_code)){
        $domain_address_code='JO';
   }
    $sqlcountry=DB::select("SELECT value,GROUP_CONCAT(scope_id SEPARATOR ',') AS `store_id`  FROM `core_config_data` WHERE path='general/country/destinations' AND value='$domain_address_code' order by config_id asc"); 
    foreach($sqlcountry as $value){
    $store_id=explode(',',$value->store_id);
    if(in_array('0',$store_id)){
     $stores=DB::select("SELECT scope_id FROM `core_config_data` WHERE path='general/country/destinations' and scope='stores' and value='$domain_address_code'");  
     $scope_id=array();
     foreach($stores as $value1){
         array_push($scope_id,$value1->scope_id);
     }
     $storesgroup=DB::select("SELECT GROUP_CONCAT(store_id SEPARATOR ',') as store_id,store_group.group_id FROM `store` INNER JOIN store_group ON store.group_id=store_group.group_id WHERE store_id!='0' group by store_group.group_id");  
     $store_id1=array();
     foreach($storesgroup as $value2){
         $store_ids=explode(',',$value2->store_id);
         foreach($store_ids as $store_ids1){
         if(in_array($store_ids1,$scope_id)){
         array_push($store_id1,$store_ids1);
         }
         }
     }
     $implode=$store_id1;
    }else{
     $implode=explode(',',$value->store_id);  
    }
    }
    $productType = isset($_GET['type']) ? $_GET['type'] : 'Grocery';
    $order_id = isset($_GET['order_id']) ? $_GET['order_id'] : '';
    if($productType == "Grocery"){
        if(!empty($order_id)){
            return $model->newQuery()->with("user")->with("orderStatus")->with('payment')->leftJoin('sales_stock', 'sales_order.entity_id', '=', 'sales_stock.order_id')->leftJoin('sales_order_item', 'sales_order.entity_id', '=', 'sales_order_item.order_id')->leftJoin('store', 'store.store_id', '=', 'sales_order.store_id')->leftJoin('store_group', 'store_group.group_id', '=', 'store.group_id')->leftJoin('order_statuses', 'order_statuses.id', '=', 'sales_order_item.order_status_id')->groupBy('sales_order.entity_id')->select('sales_order.*','order_statuses.status as orstatus','store_group.name as storename',DB::raw("SUM(sales_stock.qty) as outofstockqty"),'sales_order_item.delivery_date as delivery_date1')->whereIn('sales_order.store_id',$implode)->whereNotIn('sales_order.status',['payment_review','closed','pending'])->whereNotIn('sales_order_item.order_status_id',['12'])->where('sales_order_item.delivery_from','=','Grocery')->where('sales_order.increment_id', 'LIKE',"%{$order_id}%")->orderBy('sales_order.created_at', 'desc');
        }else{
         return $model->newQuery()->with("user")->with("orderStatus")->with('payment')->leftJoin('sales_stock', 'sales_order.entity_id', '=', 'sales_stock.order_id')->leftJoin('sales_order_item', 'sales_order.entity_id', '=', 'sales_order_item.order_id')->leftJoin('store', 'store.store_id', '=', 'sales_order.store_id')->leftJoin('store_group', 'store_group.group_id', '=', 'store.group_id')->leftJoin('order_statuses', 'order_statuses.id', '=', 'sales_order_item.order_status_id')->groupBy('sales_order.entity_id')->select('sales_order.*','order_statuses.status as orstatus','store_group.name as storename',DB::raw("SUM(sales_stock.qty) as outofstockqty"),'sales_order_item.delivery_date as delivery_date1')->whereIn('sales_order.store_id',$implode)->whereNotIn('sales_order.status',['payment_review','closed','pending'])->whereNotIn('sales_order_item.order_status_id',['12'])->where('sales_order_item.delivery_from','=','Grocery')->orderBy('sales_order.created_at', 'desc');   
        }    
            
    } else if($productType == "Warehouse") {
        
         if(!empty($order_id)){
           return $model->newQuery()->with("user")->with("orderStatus")->with('payment')->leftJoin('sales_stock', 'sales_order.entity_id', '=', 'sales_stock.order_id')->leftJoin('sales_order_item', 'sales_order.entity_id', '=', 'sales_order_item.order_id')->leftJoin('store', 'store.store_id', '=', 'sales_order.store_id')->leftJoin('store_group', 'store_group.group_id', '=', 'store.group_id')->leftJoin('order_statuses', 'order_statuses.id', '=', 'sales_order_item.order_status_id')->groupBy('sales_order.entity_id')->select('sales_order.*','order_statuses.status as orstatus','store_group.name as storename',DB::raw("SUM(sales_stock.qty) as outofstockqty"),'sales_order_item.delivery_date as delivery_date1')->whereIn('sales_order.store_id',$implode)->whereNotIn('sales_order.status',['payment_review','closed','pending'])->whereNotIn('sales_order_item.order_status_id',['12'])->where('sales_order_item.delivery_from','=','Warehouse')->where('sales_order.increment_id', 'LIKE',"%{$order_id}%")->orderBy('sales_order.created_at', 'desc');   
         }else{
        return $model->newQuery()->with("user")->with("orderStatus")->with('payment')->leftJoin('sales_stock', 'sales_order.entity_id', '=', 'sales_stock.order_id')->leftJoin('sales_order_item', 'sales_order.entity_id', '=', 'sales_order_item.order_id')->leftJoin('store', 'store.store_id', '=', 'sales_order.store_id')->leftJoin('store_group', 'store_group.group_id', '=', 'store.group_id')->leftJoin('order_statuses', 'order_statuses.id', '=', 'sales_order_item.order_status_id')->groupBy('sales_order.entity_id')->select('sales_order.*','order_statuses.status as orstatus','store_group.name as storename',DB::raw("SUM(sales_stock.qty) as outofstockqty"),'sales_order_item.delivery_date as delivery_date1')->whereIn('sales_order.store_id',$implode)->whereNotIn('sales_order.status',['payment_review','closed','pending'])->whereNotIn('sales_order_item.order_status_id',['12'])->where('sales_order_item.delivery_from','=','Warehouse')->orderBy('sales_order.created_at', 'desc');
         }
    } else if($productType == "Supplier"){
        if(!empty($order_id)){
          return $model->newQuery()->with("user")->with("orderStatus")->with('payment')->leftJoin('sales_stock', 'sales_order.entity_id', '=', 'sales_stock.order_id')->leftJoin('sales_order_item', 'sales_order.entity_id', '=', 'sales_order_item.order_id')->leftJoin('store', 'store.store_id', '=', 'sales_order.store_id')->leftJoin('store_group', 'store_group.group_id', '=', 'store.group_id')->leftJoin('order_statuses', 'order_statuses.id', '=', 'sales_order_item.order_status_id')->groupBy('sales_order.entity_id')->select('sales_order.*','order_statuses.status as orstatus','store_group.name as storename',DB::raw("SUM(sales_stock.qty) as outofstockqty"),'sales_order_item.delivery_date as delivery_date1')->whereIn('sales_order.store_id',$implode)->whereNotIn('sales_order.status',['payment_review','closed','pending'])->whereNotIn('sales_order_item.order_status_id',['12'])->where('sales_order_item.delivery_from','=','Supplier')->where('sales_order.increment_id', 'LIKE',"%{$order_id}%")->orderBy('sales_order.created_at', 'desc');    
        }else{
        return $model->newQuery()->with("user")->with("orderStatus")->with('payment')->leftJoin('sales_stock', 'sales_order.entity_id', '=', 'sales_stock.order_id')->leftJoin('sales_order_item', 'sales_order.entity_id', '=', 'sales_order_item.order_id')->leftJoin('store', 'store.store_id', '=', 'sales_order.store_id')->leftJoin('store_group', 'store_group.group_id', '=', 'store.group_id')->leftJoin('order_statuses', 'order_statuses.id', '=', 'sales_order_item.order_status_id')->groupBy('sales_order.entity_id')->select('sales_order.*','order_statuses.status as orstatus','store_group.name as storename',DB::raw("SUM(sales_stock.qty) as outofstockqty"),'sales_order_item.delivery_date as delivery_date1')->whereIn('sales_order.store_id',$implode)->whereNotIn('sales_order.status',['payment_review','closed','pending'])->whereNotIn('sales_order_item.order_status_id',['12'])->where('sales_order_item.delivery_from','=','Supplier')->orderBy('sales_order.created_at', 'desc');
        }
    }
          
        } else if (auth()->user()->hasRole('manager')) {
           
            // return $model->newQuery()->with("user")->with("orderStatus")->with('payment')
            //     ->join("product_orders", "orders.id", "=", "product_orders.order_id")
            //     ->join("products", "products.id", "=", "product_orders.product_id")
            //     ->join("user_markets", "user_markets.market_id", "=", "products.market_id")
            //     ->leftJoin('sales_stock', 'sales_order.entity_id', '=', 'sales_stock.order_id')
            //     ->where('user_markets.user_id', auth()->id())
            //     ->groupBy('orders.id')
            //     ->select('orders.*',DB::raw("SUM(sales_stock.qty) as outofstockqty"));
            //  return $model->newQuery()->with("user")->with("orderStatus")->with('payment')->leftJoin('sales_stock', 'sales_order.entity_id', '=', 'sales_stock.order_id')->groupBy('sales_order.entity_id')->select('sales_order.*',DB::raw("SUM(sales_stock.qty) as outofstockqty"));
             return $model->newQuery()->with("user")->with("orderStatus")->with('payment')->leftJoin('sales_stock', 'sales_order.entity_id', '=', 'sales_stock.order_id')->leftJoin('store_group', 'store_group.group_id', '=', 'sales_order.store_id')->leftJoin('order_statuses', 'order_statuses.id', '=', 'sales_order.order_status_id')->groupBy('sales_order.entity_id')->select('sales_order.*','order_statuses.status as orstatus','store_group.name as storename',DB::raw("SUM(sales_stock.qty) as outofstockqty"));
        } else if (auth()->user()->hasRole('client')) {
            return $model->newQuery()->with("user")->with("orderStatus")->with('payment')
            ->leftJoin('sales_stock', 'sales_order.entity_id', '=', 'sales_stock.order_id')
                ->where('orders.user_id', auth()->id())
                ->groupBy('orders.id')
                ->select('orders.*',DB::raw("SUM(sales_stock.qty) as outofstockqty"));
        } else if (auth()->user()->hasRole('driver')) {
            return $model->newQuery()->with("user")->with("orderStatus")->with('payment')
            ->leftJoin('sales_stock', 'sales_order.entity_id', '=', 'sales_stock.order_id')
                ->where('sales_order.driver_id', auth()->id())
                ->groupBy('sales_order.entity_id')
                ->select('sales_order.*',DB::raw("SUM(sales_stock.qty) as outofstockqty"));
        } else {
             return $model->newQuery()->with("user")->with("orderStatus")->with('payment')->leftJoin('sales_stock', 'sales_order.entity_id', '=', 'sales_stock.order_id')->leftJoin('sales_order_item', 'sales_order.entity_id', '=', 'sales_order_item.order_id')->leftJoin('store', 'store.store_id', '=', 'sales_order.store_id')->leftJoin('store_group', 'store_group.group_id', '=', 'store.group_id')->leftJoin('order_statuses', 'order_statuses.id', '=', 'sales_order_item.order_status_id')->groupBy('sales_order.entity_id')->select('sales_order.*','order_statuses.status as orstatus','store_group.name as storename',DB::raw("SUM(sales_stock.qty) as outofstockqty"),'sales_order_item.delivery_date as delivery_date1')->whereIn('sales_order.store_id',$implode)->whereNotIn('sales_order.status',['payment_review','closed','pending'])->where('sales_order_item.delivery_from','=','Grocery')->orderBy('sales_order.created_at', 'desc');
        }

    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\DataTables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            ->minifiedAjax()
            ->addAction(['title'=>trans('lang.actions'),'width' => '80px', 'printable' => false, 'responsivePriority' => '100'])
            ->parameters(array_merge(
                [
                    'language' => json_decode(
                        file_get_contents(base_path('resources/lang/' . app()->getLocale() . '/datatable.json')
                        ), true),
                    'order' => [ [0, 'desc'] ],
                ],
                config('datatables-buttons.parameters')
            ));
    }

    /**
     * Export PDF using DOMPDF
     * @return mixed
     */
    public function pdf()
    {
        $data = $this->getDataForPrint();
        $pdf = PDF::loadView($this->printPreview, compact('data'));
        return $pdf->download($this->filename() . '.pdf');
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'ordersdatatable_' . time();
    }
}