<?php
namespace App\Http\Controllers\API;

use App\Criteria\Orders\OrdersOfStatusesCriteria;
use App\Criteria\Orders\OrdersOfUserCriteria;
use App\Events\OrderChangedEvent;
use App\Http\Controllers\Controller;
use App\Models\Order;
use App\Notifications\AssignedOrder;
use App\Notifications\NewOrder;
use App\Notifications\StatusChangedOrder;
use App\Repositories\CartRepository;
use App\Repositories\NotificationRepository;
use App\Repositories\OrderRepository;
use App\Repositories\PaymentRepository;
use App\Repositories\ProductOrderRepository;
use App\Repositories\UserRepository;
use Flash;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Notification;
use InfyOm\Generator\Criteria\LimitOffsetCriteria;
use Prettus\Repository\Criteria\RequestCriteria;
use Prettus\Repository\Exceptions\RepositoryException;
use Prettus\Validator\Exceptions\ValidatorException;
use Stripe\Token;
use DB;
use Magento\Framework\App\Bootstrap;
/**
 * Class OrderController
 * @package App\Http\Controllers\API
 */
class OrderAPIController extends Controller
{
    /** @var  OrderRepository */
    private $orderRepository;
    /** @var  ProductOrderRepository */
    private $productOrderRepository;
    /** @var  CartRepository */
    private $cartRepository;
    /** @var  UserRepository */
    private $userRepository;
    /** @var  PaymentRepository */
    private $paymentRepository;
    /** @var  NotificationRepository */
    private $notificationRepository;

    /**
     * OrderAPIController constructor.
     * @param OrderRepository $orderRepo
     * @param ProductOrderRepository $productOrderRepository
     * @param CartRepository $cartRepo
     * @param PaymentRepository $paymentRepo
     * @param NotificationRepository $notificationRepo
     * @param UserRepository $userRepository
     */
    public function __construct(OrderRepository $orderRepo, ProductOrderRepository $productOrderRepository, CartRepository $cartRepo, PaymentRepository $paymentRepo, NotificationRepository $notificationRepo, UserRepository $userRepository)
    {
        $this->orderRepository = $orderRepo;
        $this->productOrderRepository = $productOrderRepository;
        $this->cartRepository = $cartRepo;
        $this->userRepository = $userRepository;
        $this->paymentRepository = $paymentRepo;
        $this->notificationRepository = $notificationRepo;
    }

    /**
     * Display a listing of the Order.
     * GET|HEAD /orders
     *
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */
    public function index(Request $request)
    {
        try {
            $this->orderRepository->pushCriteria(new RequestCriteria($request));
            $this->orderRepository->pushCriteria(new LimitOffsetCriteria($request));
            $this->orderRepository->pushCriteria(new OrdersOfStatusesCriteria($request));
            $this->orderRepository->pushCriteria(new OrdersOfUserCriteria(auth()->id()));
        } catch (RepositoryException $e) {
            return $this->sendError($e->getMessage());
        }
        $orders = $this->orderRepository->all();

        return $this->sendResponse($orders->toArray(), 'Orders retrieved successfully');
    }

    /**
     * Display the specified Order.
     * GET|HEAD /orders/{id}
     *
     * @param int $id
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function show(Request $request, $id)
    {
        /** @var Order $order */
        if (!empty($this->orderRepository)) {
            try {
                $this->orderRepository->pushCriteria(new RequestCriteria($request));
                $this->orderRepository->pushCriteria(new LimitOffsetCriteria($request));
                $this->orderRepository->pushCriteria(new OrdersOfStatusesCriteria($request));
                $this->orderRepository->pushCriteria(new OrdersOfUserCriteria(auth()->id()));
            } catch (RepositoryException $e) {
                return $this->sendError($e->getMessage());
            }
            $order = $this->orderRepository->findWithoutFail($id);

            //Sub Status Check Start
            $entity_id = $id;
            $ProType = isset($_GET["type"]) ? $_GET["type"] : "Grocery";
            $resullttss1 = DB::select("SELECT status FROM sales_order WHERE entity_id='$entity_id'");
            $currentMainStatus = isset($resullttss1[0]->status) ? $resullttss1[0]->status : "";
            $resulltts = DB::select("SELECT sub_status FROM sales_order_item WHERE order_id='$entity_id' and product_type='simple' and delivery_from='$ProType'");
            $aar = array();
            foreach ($resulltts as $resultss1) {
                if (empty($resultss1->sub_status) || is_null($resultss1->sub_status)) {
                    $subStatus = '';
                    if ($currentMainStatus == "processing") {
                        $subStatus = $currentMainStatus;
                    } else {
                        $subStatus = "Order Placed";
                    }
                    array_push($aar, $subStatus);
                } else {
                    array_push($aar, $resultss1->sub_status);
                }
            }
            // print_r($aar);
            $subStatusArrayCount = array_count_values($aar);
            $stCheck = '';
            if (($currentMainStatus == 'pending') || ($currentMainStatus == 'canceled')) {
                $stCheck = $currentMainStatus;
            } else {
                if (count($subStatusArrayCount) > 1) {
                    foreach ($subStatusArrayCount as $key => $subStatusArrayCount1) {
                        if ($key != "canceled") {
                            $stCheck = $key;
                        }
                    }
                } else {
                    $stCheck = key($subStatusArrayCount);
                }
            }
            $order1 = $order->toArray();
            $order1["order_status"]["current_sub_status"] = $stCheck;
            //Sub Status Check End

            //Coupon Code Check Start
            $resultCoupon = DB::table('sales_order')->select('coupon_code', 'discount_amount')->where('entity_id', $entity_id)->first();
            $discountVal = isset($resultCoupon->coupon_code) ? abs($resultCoupon->discount_amount) : 0;
            $couponVal = isset($resultCoupon->coupon_code) ? $resultCoupon->coupon_code : null;
            $order1["coupon_code_value"] = $couponVal;
            $order1["coupon_redemed_amount"] = "$discountVal";
            if ($couponVal != "" && $couponVal != null) {
                $order1["coupon_code_active"] = 1;
            } else {
                $order1["coupon_code_active"] = 0;
            }
            //Coupon Code Check End
        }

        if (empty($order)) {
            return $this->sendError('Order not found');
        }

        // return $this->sendResponse($order->toArray(), 'Order retrieved successfully');
        return $this->sendResponse($order1, 'Order retrieved successfully');
    }

    /**
     * Store a newly created Order in storage.
     *
     * @param Request $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function store(Request $request)
    {
        $payment = $request->only('payment');
        if (isset($payment['payment']) && $payment['payment']['method']) {
            if ($payment['payment']['method'] == "Credit Card (Stripe Gateway)") {
                return $this->stripPayment($request);
            } else {
                return $this->cashPayment($request);

            }
        }
    }

    /**
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|mixed
     */
    private function stripPayment(Request $request)
    {
        $input = $request->all();
        $amount = 0;
        try {
            $user = $this->userRepository->findWithoutFail($input['user_id']);
            if (empty($user)) {
                return $this->sendError('User not found');
            }
            $stripeToken = Token::create(array(
                "card" => array(
                    "number" => $input['stripe_number'],
                    "exp_month" => $input['stripe_exp_month'],
                    "exp_year" => $input['stripe_exp_year'],
                    "cvc" => $input['stripe_cvc'],
                    "name" => $user->name,
                )
            ));
            if ($stripeToken->created > 0) {
                if (empty($input['delivery_address_id'])) {
                    $order = $this->orderRepository->create(
                        $request->only('user_id', 'order_status_id', 'tax', 'hint')
                    );
                } else {
                    $order = $this->orderRepository->create(
                        $request->only('user_id', 'order_status_id', 'tax', 'delivery_address_id', 'delivery_fee', 'hint')
                    );
                }
                foreach ($input['products'] as $productOrder) {
                    $productOrder['order_id'] = $order->id;
                    $amount += $productOrder['price'] * $productOrder['quantity'];
                    $this->productOrderRepository->create($productOrder);
                }
                $amount += $order->delivery_fee;
                $amountWithTax = $amount + ($amount * $order->tax / 100);
                $charge = $user->charge((int) ($amountWithTax * 100), ['source' => $stripeToken]);
                $payment = $this->paymentRepository->create([
                    "user_id" => $input['user_id'],
                    "description" => trans("lang.payment_order_done"),
                    "price" => $amountWithTax,
                    "status" => $charge->status, // $charge->status
                    "method" => $input['payment']['method'],
                ]);
                $this->orderRepository->update(['payment_id' => $payment->id], $order->id);

                $this->cartRepository->deleteWhere(['user_id' => $order->user_id]);

                Notification::send($order->productOrders[0]->product->market->users, new NewOrder($order));
            }
        } catch (ValidatorException $e) {
            return $this->sendError($e->getMessage());
        }

        return $this->sendResponse($order->toArray(), __('lang.saved_successfully', ['operator' => __('lang.order')]));
    }

    /**
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|mixed
     */
    private function cashPayment(Request $request)
    {
        $input = $request->all();
        $amount = 0;
        try {
            $order = $this->orderRepository->create(
                $request->only('user_id', 'order_status_id', 'tax', 'delivery_address_id', 'delivery_fee', 'hint')
            );
            Log::info($input['products']);
            foreach ($input['products'] as $productOrder) {
                $productOrder['order_id'] = $order->id;
                $amount += $productOrder['price'] * $productOrder['quantity'];
                $this->productOrderRepository->create($productOrder);
            }
            $amount += $order->delivery_fee;
            $amountWithTax = $amount + ($amount * $order->tax / 100);
            $payment = $this->paymentRepository->create([
                "user_id" => $input['user_id'],
                "description" => trans("lang.payment_order_waiting"),
                "price" => $amountWithTax,
                "status" => 'Waiting for Client',
                "method" => $input['payment']['method'],
            ]);

            $this->orderRepository->update(['payment_id' => $payment->id], $order->id);

            $this->cartRepository->deleteWhere(['user_id' => $order->user_id]);

            Notification::send($order->productOrders[0]->product->market->users, new NewOrder($order));

        } catch (ValidatorException $e) {
            return $this->sendError($e->getMessage());
        }

        return $this->sendResponse($order->toArray(), __('lang.saved_successfully', ['operator' => __('lang.order')]));
    }

    /**
     * Update the specified Order in storage.
     *
     * @param int $id
     * @param Request $request
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function update($id, Request $request)
    {
        $oldOrder = $this->orderRepository->findWithoutFail($id);
        if (empty($oldOrder)) {
            return $this->sendError('Order not found');
        }
        //$oldStatus = $oldOrder->payment->status;
        $input = $request->all();
        date_default_timezone_set("Asia/Amman");
        try {
            $rec = DB::table('core_config_data')->where('path', '=', 'web/secure/base_url')->first();
            $baseSiteUrl = isset($rec->value) ? $rec->value : '';

            $order = $this->orderRepository->update($input, $id);
            $order_status_id = $input['order_status_id'];
            $product_id = isset($input['product_id']) ? $input['product_id'] : '';
            //print_r($product_id);
            //echo "fgfgj"; exit;
            $increment = DB::table('sales_order')->select('increment_id', 'customer_id', 'grand_total', 'store_id')->where('entity_id', '=', $id)->get();
            $increment_id = isset($increment[0]->increment_id) ? $increment[0]->increment_id : '';
            $customer_id = isset($increment[0]->customer_id) ? $increment[0]->customer_id : '';
            $store_id = isset($increment[0]->store_id) ? $increment[0]->store_id : '';
            // $grand_total=isset($increment[0]->grand_total) ? $increment[0]->grand_total : ''; 
            $langg = "en";
            if (!empty($store_id)) {
                $stores = DB::table('store')->select('code')->where('store_id', '=', $store_id)->get();
                $store_code = isset($stores[0]->code) ? $stores[0]->code : '';
                $code = explode("_", $store_code);
                $langg = isset($code) ? end($code) : 'en';
            }
            $state = "processing";
            $order_details_block = 0;
            $checkinbag = 1;

            $not_delivered = 0;
            // if($order_status_id=="5"){
// $ngenius = DB::table('ngenius_networkinternational')->where('order_id','=',$increment_id)->get();
// $ngeniuspayment = DB::table('sales_order_payment')->where('parent_id','=',$id)->where('method','=','ngeniusonline')->get();
// if(count($ngenius)>0 && count($ngeniuspayment)>0){
// $amount=isset($increment[0]->grand_total) ? $increment[0]->grand_total : '';  
// $reference=isset($ngenius[0]->reference) ? $ngenius[0]->reference : ''; 
// $payment_id=isset($ngenius[0]->payment_id) ? $ngenius[0]->payment_id : ''; 
// $amount1=$amount * 100;

            // $apikey_ref= DB::table('core_config_data')->where('path','=','payment/ngeniusonline/api_key')->get();
// $apikey=$apikey_ref[0]->value;   
// $outlet_ref= DB::table('core_config_data')->where('path','=','payment/ngeniusonline/outlet_ref')->get();
// $outlet=$outlet_ref[0]->value; // enter your API key here
// $environment_ref= DB::table('core_config_data')->where('path','=','payment/ngeniusonline/environment')->get();
// $environment=$environment_ref[0]->value;
// if($environment=="uat"){
//  $paymenturl="https://api-gateway.sandbox.ngenius-payments.com/";   
// }else{
//  $paymenturl="https://api-gateway.ngenius-payments.com/";   
// }
// $ch = curl_init(); 
// curl_setopt($ch,CURLOPT_URL,$paymenturl."identity/auth/access-token"); 
// curl_setopt($ch,CURLOPT_HTTPHEADER,array(
//     "accept: application/vnd.ni-identity.v1+json",
//     "authorization: Basic ".$apikey,
//     "content-type: application/vnd.ni-identity.v1+json"
//   )); 
// curl_setopt($ch,CURLOPT_RETURNTRANSFER,true);   
// curl_setopt($ch,CURLOPT_POST, 1); 
// if($environment=="uat"){
// curl_setopt($ch, CURLOPT_POSTFIELDS,  "{\"realmName\":\"ni\"}"); 
// }else{
// curl_setopt($ch, CURLOPT_POSTFIELDS,  "{\"realmName\":\"networkinternational\"}");    
// }
// $output = json_decode(curl_exec($ch)); 
// $access_token = $output->access_token;

            // $token=$access_token; 

            // $ch = curl_init(); 
// $json=array("amount"=>array("currencyCode"=>"AED","value"=>"$amount1"));
// curl_setopt($ch, CURLOPT_URL,$paymenturl."transactions/outlets/".$outlet."/orders/".$reference."/payments/".$payment_id."/captures"); 
// curl_setopt($ch, CURLOPT_HTTPHEADER, array(
//     "Authorization: Bearer ".$token, 
//     "Content-Type: application/vnd.ni-payment.v2+json", 
//     "Accept: application/vnd.ni-payment.v2+json")); 
// curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);   
// curl_setopt($ch, CURLOPT_POST, 1); 
// curl_setopt($ch, CURLOPT_POSTFIELDS,json_encode($json)); 

            // $output = json_decode(curl_exec($ch));  
// //print_r($output);
// $capture=isset($output->state) ? $output->state : '';
// if(!empty($capture)){
// DB::table('ngenius_networkinternational')->where('order_id','=',$increment_id)->update(['status'=>"ngenius_complete",'state'=>"CAPTURED",'captured_amt'=>$amount]);   
// }else{
//  $not_delivered=1;    
// }
// curl_close ($ch);    
// }
// }

            if ($order_status_id == "5") {
                $qatarpayment = DB::table('sales_order_payment')->where('parent_id', '=', $id)->where('method', '=', 'tns_hosted')->get();
                if (count($qatarpayment) > 0) {
                    $amount = isset($increment[0]->grand_total) ? $increment[0]->grand_total : '';
                    $merchentId = "9511489223EP";
                    //$merchentId="TESTAHIQNB01";
                    $ch = curl_init();
                    curl_setopt($ch, CURLOPT_URL, "https://ap-gateway.mastercard.com/api/rest/version/62/merchant/$merchentId/order/$increment_id");
                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                    //curl_setopt($ch, CURLOPT_GET, 1);
                    $apikey = "bWVyY2hhbnQuOTUxMTQ4OTIyM0VQOjk0M2RkOGFiYTE0MzRiNWQyMGRiYzNiYWQ5MjZhNGMz";
                    //$apikey="bWVyY2hhbnQuVEVTVEFISVFOQjAxOjY5MzZhODk5MGIxMWFiNjA3MGY1MTc3MGVlYmFiNDRi";
                    curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                        "authorization: Basic " . $apikey,
                        "content-type: application/json"
                    ));
                    $results = curl_exec($ch);
                    if (curl_errno($ch)) {
                        echo "Error:" . curl_errno($ch);
                    }
                    curl_close($ch);
                    $output = json_decode($results, true);
                    $result = $output['result'];
                    if ($result == "SUCCESS") {
                        $status = $output['status'];
                        if ($status == "AUTHORIZED") {
                            $ch = curl_init();
                            curl_setopt($ch, CURLOPT_URL, "https://ap-gateway.mastercard.com/api/rest/version/62/merchant/$merchentId/order/$increment_id/transaction/$increment_id");
                            $amount = number_format((float) $amount, 3, '.', '');
                            $json = array("apiOperation" => "CAPTURE", "transaction" => array("amount" => $amount, "currency" => "JOD"));
                            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT");
                            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                                "authorization: Basic " . $apikey,
                                "content-type: application/json"
                            ));
                            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($json));
                            $results = curl_exec($ch);
                            if (curl_errno($ch)) {
                                echo "Error:" . curl_errno($ch);
                            }
                            curl_close($ch);
                            $output1 = json_decode($results, true);
                            $result1 = $output1['result'];
                            print_r($output1);
                            if ($result1 == "SUCCESS") {
                                $not_delivered = 0;
                            } else {
                                $not_delivered = 1;
                            }
                        }
                        if ($status == "PARTIALLY_CAPTURED") {

                            $increment_id1 = $increment_id . '_1';

                            $product_item = DB::table('sales_order_item')->where('order_id', '=', $id)->where('delivery_from', '=', 'Grocery')->get();
                            if (count($product_item) > 0) {
                                $finalamount = 0;
                                foreach ($product_item as $product_item1) {
                                    $finalamount += $product_item1->row_total - $product_item1->discount_amount;
                                }
                                $finalamount = number_format((float) $finalamount, 2, '.', '');
                                $ch = curl_init();
                                curl_setopt($ch, CURLOPT_URL, "https://ap-gateway.mastercard.com/api/rest/version/62/merchant/$merchentId/order/$increment_id/transaction/$increment_id1");
                                $json = array("apiOperation" => "CAPTURE", "transaction" => array("amount" => $finalamount, "currency" => "JOD"));
                                curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                                curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT");
                                curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                                    "authorization: Basic " . $apikey,
                                    "content-type: application/json"
                                ));
                                curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($json));
                                $results = curl_exec($ch);
                                if (curl_errno($ch)) {
                                    echo "Error:" . curl_errno($ch);
                                }
                                curl_close($ch);
                                $output1 = json_decode($results, true);
                                $result1 = $output1['result'];
                                if ($result1 == "SUCCESS") {
                                    $not_delivered = 0;
                                } else {
                                    $not_delivered = 1;
                                }
                            } else {
                                $not_delivered = 1;
                            }

                        }
                    }
                }
            }
            if ($not_delivered == 0) {
                $message = "";
                if ($order_status_id == "2") {
                    $status = "startpicking";
                    $order_details_block = 1;
                    $checkinbag = 1;
                    if ($langg == "en") {
                        $message = "Dear customer, your order ID: #$increment_id has been successfully start on picking.         عميلنا الكريم, طلبكم قيد التحضير";
                    } else if ($langg == "ar") {
                        $message = "Your Order ID:#$increment_id has been successfully Start on picking";
                    }
                } else if ($order_status_id == "3") {
                    $url1 = $baseSiteUrl . '/api/weighteditem.php?id=' . $id . '';
                    $headers1 = array();
                    $headers1[] = 'Content-Type: application/json';
                    $ch1 = curl_init();
                    curl_setopt($ch1, CURLOPT_URL, $url1);
                    curl_setopt($ch1, CURLOPT_CUSTOMREQUEST, "GET");
                    curl_setopt($ch1, CURLOPT_HTTPHEADER, $headers1);
                    curl_setopt($ch1, CURLOPT_RETURNTRANSFER, true);
                    //Send the request
                    $response1 = curl_exec($ch1);
                    //Close request
                    curl_close($ch1);
                    $status = "endpicking";
                    if ($langg == "en") {
                        $message = "Your Order ID:#$increment_id has been successfully End picking.   عميلنا الكريم, تم الانتهاء من تحضير طلبكم";
                    } else if ($langg == "ar") {
                        $message = "Your Order ID:#$increment_id has been successfully End picking";
                    }
                    $checkinbag = 1;
                    $checkaction = DB::table('sales_order_item')->select('order_id')->where('order_id', '=', $id)->where('actionrequired', '=', '1')->get();
                    if (count($checkaction) > 0) {
                        $status = "actionrequired";
                        if ($langg == "en") {
                            $message = "Your Order ID:#$increment_id has an issue, hence please contact support.";
                        } else if ($langg == "ar") {
                            $message = "Your Order ID:#$increment_id has an issue, hence please contact support.";
                        }
                    }

                    //  $checktotalqty=DB::table('sales_order_item')->select('order_id')->where('order_id','=',$id)->get();
                    //  $checkbagqty=DB::table('add_to_bag')->select('order_id')->where('order_id','=',$id)->get();
                    //  if(count($checktotalqty)!=count($checkbagqty)){
                    //   $checkinbag=0;  
                    //  }

                } else if ($order_status_id == "4") {
                    $status = "ontheway";
                    if ($langg == "en") {
                        $message = "Your Order ID:#$increment_id has been successfully on the way    طلبكم في الطريق اليكم";
                    } else if ($langg == "ar") {
                        $message = "Your Order ID:#$increment_id has been successfully Ontheway.";
                    }
                    $checkinbag = 1;
                } else if ($order_status_id == "5") {
                    $status = "delivered";
                    if ($langg == "en") {
                        //$message = "Dear customer, Thank you for shopping with us! Check your YAHALA points and rate your experience below.
//<a href='https://onelink.to/ctow'>https://onelink.to/ctow</a>       شكرًا لتسوقكم معنا! يمكنكم الآن التحقق من نقاط ياهلا وتقييم تجربتكم عبر الرابط في الاعلى";
                        $message="Dear Customer, Thank you for shopping with us, You can now check YAHALA points.                         شكراً لتسوقكم معنا، بإمكانكم الآن التحقق من نقاط ياهلا.";          
                        //$message="Dear customer, your order ID:#$increment_id has been successfully  Delivered. Thank you for shopping with us, you can now check your YAHAlA points. 
//تم توصيل طلبكم. شكراً لتسوقكم معنا, بإمكانكم الآن التحقق من نقاط ياهلا ";  


                    } else if ($langg == "ar") {
                        $message = "Your Order ID:#$increment_id has been successfully Delivered.";
                    }
                    $checkinbag = 1;
                } else if ($order_status_id == "6") {
                    $status = "actionrequired";
                    if ($langg == "en") {
                        $message = "Your Order ID:#$increment_id has an issue, hence please contact support.";
                    } else if ($langg == "ar") {
                        $message = "Your Order ID:#$increment_id has an issue, hence please contact support.";
                    }
                    $checkinbag = 1;
                } else if ($order_status_id == "7") {
                    $status = "canceled";
                    // $message="Your Order ID:#$increment_id has been cancelled."; 
                    if ($langg == "en") {
                        $message = "Hello there, we are sorry to inform you that your order has been canceled.";
                    } else if ($langg == "ar") {
                        $message = "Hello there, we are sorry to inform you that your order has been canceled.";
                    }
                    $checkinbag = 1;
                } else if ($order_status_id == "8") {
                    $status = "readytodeliver";
                    // $message="Your Order ID:#$increment_id has been cancelled."; 
                    if ($langg == "en") {
                        $message = "Your Order ID:#$increment_id has been Ready to Deliver. Collect to Store";
                    } else if ($langg == "ar") {
                        $message = "Your Order ID:#$increment_id has been Ready to Deliver. Collect to Store.";
                    }
                    $checkinbag = 1;
                }

                if ($checkinbag == 1) {
                    $created = date("Y-m-d H:i:s");

                    $customer = DB::table('sales_order')->select('customer_entity_varchar.value')->join('customer_entity_varchar', 'customer_entity_varchar.entity_id', '=', 'sales_order.customer_id')->where('sales_order.entity_id', '=', $id)->where('attribute_id', '=', '193')->get();
                    $phone_number = isset($customer[0]->value) ? $customer[0]->value : '';
                    //$substr=substr($phone_number, 0, 3); 
// if($substr=="971"){
//   $phone_number1=$phone_number;  
// }else{
//  $phone_number1="971".$phone_number;
// }

                    // $phonemno ='["'.$phone_number.'"]';
// $msg = '"'.$message.'"';

                    // $getcountry = DB::table('core_config_data')->select('value','GROUP_CONCAT(scope_id SEPARATOR ',') AS store_id')->where('path','=','general/country/destinations')->whereIn('scope_id',$store_id)->get();
                    $getcountry = DB::select("SELECT value,GROUP_CONCAT(scope_id SEPARATOR ',') AS store_id FROM core_config_data WHERE scope_id IN($store_id) AND path='general/country/destinations'");
                    // print_r($getcountry);
// exit;
                    $current_country = isset($getcountry[0]->value) ? $getcountry[0]->value : 'JO';
                    $url = "";
                    //Sms Sending For user
                    if ($current_country == "JO") {
                        $curl = curl_init();
                        curl_setopt_array($curl, array(
                            CURLOPT_URL => $baseSiteUrl . '/api/orderstatusmsgsend.php?nocache=' . time(),
                            CURLOPT_RETURNTRANSFER => true,
                            CURLOPT_ENCODING => '',
                            CURLOPT_MAXREDIRS => 10,
                            CURLOPT_TIMEOUT => 0,
                            CURLOPT_FOLLOWLOCATION => true,
                            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                            CURLOPT_CUSTOMREQUEST => 'POST',
                            CURLOPT_POSTFIELDS => array('phone_number' => $phone_number, 'msg' => $message),
                        ));

                        $response = curl_exec($curl);
                        $err = curl_error($curl);
                        curl_close($curl);

                        // if ($err) {
                        //  echo "error";
                        // } else {
                        //  echo "success";   
                        // }
                    }

                    $url = 'https://fcm.googleapis.com/fcm/send';

                    $devicetoken = DB::table('customer_devices')->select('token')->where('user_id', '=', $customer_id)->get();
                    $token = isset($devicetoken[0]->token) ? $devicetoken[0]->token : '';
                    $serverKey = "AAAApll5Gkw:APA91bEyUBsBXnBFY0ANjg7vWa-nSyPUeBZxCcABeLxV4aNEvpxtUTDKGCBGP4nuYENxE07r72FCIaC0TNaUD3F8ALgtsuud1raEE1BkTZrDiaqOjd3tnhn-JTZbu3hPGrdNR_H3m1BT";
                    $title = "CTown";
                    $notification = array('title' => $title, 'body' => $message, 'sound' => '1', 'badge' => '1');
                    $additionalData = array("click_action" => "FLUTTER_NOTIFICATION_CLICK", 'status' => "done");
                    $arrayToSend = array('to' => $token, 'notification' => $notification, 'priority' => 'high', "data" => $additionalData);
                    $json = json_encode($arrayToSend);
                    $headers = array();
                    $headers[] = 'Content-Type: application/json';
                    $headers[] = 'Authorization: key=' . $serverKey;
                    $ch = curl_init();
                    curl_setopt($ch, CURLOPT_URL, $url);
                    curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
                    curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
                    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                    //Send the request
                    $response = curl_exec($ch);
                    //Close request
                    curl_close($ch);

                    // Customer web notification start
                    $cdate = date('Y-m-d');
                    $created_date_time = gmdate("Y-m-d H:i:s");
                    // DB::table('customer_notification')->insert(
//     ['user_id' => "$id", 'title' => "$title"]
// );
                    DB::statement("INSERT INTO customer_notification(user_id, title, message, created_date, created_date_time) VALUES('$customer_id', '$title', '$message', '$cdate', '$created_date_time')");
                    DB::statement("INSERT INTO customer_notification1(notify_alert, user_id) VALUES('1', '$customer_id')");
                    // Customer web notification end

                    foreach ($product_id as $key => $value) {
                        DB::table('sales_order_item')->where('order_id', '=', $id)->where('product_id', '=', $value)->update(['order_status_id' => $order_status_id, 'sub_status' => $status]);

                        $getconfig = DB::table('sales_order_item')->select('sku')->where('order_id', '=', $id)->where('product_id', '=', $value)->get();
                        if (count($getconfig) > 0) {
                            $configsku = isset($getconfig[0]->sku) ? $getconfig[0]->sku : '';
                            DB::table('sales_order_item')->where('order_id', '=', $id)->where('sku', '=', $configsku)->where('product_type', '=', 'configurable')->update(['order_status_id' => $order_status_id, 'sub_status' => $status]);
                        }

                    }

                    $getproductcount = DB::table('sales_order_item')->select('order_id')->where('order_id', '=', $id)->get();

                    $getproductcount1 = DB::table('sales_order_item')->select('order_id')->where('order_id', '=', $id)->where('sub_status', '=', $status)->get();


                    // $returnbtnenable=DB::table('sales_order_item')->select('order_status_id', 'sub_status', 'product_id')->where('order_id','=',$id)->where('sub_status','=','delivered')->get();

                    // foreach($returnbtnenable as $key1=>$value1){
//      DB::table('sales_order_item')->where('order_id','=',$id)->where('product_id','=',$value1->product_id)->update(['updated_at'=>$created_date_time]);
// }


                    //Local Notifcation For Mobile Start
                    date_default_timezone_set('Asia/Amman');
                    $updated_at = date("Y-m-d H:i:s");
                    $dateonly = date('Y-m-d');

                    DB::table('mobile_customer_notification')->insert([
                        'user_id' => $customer_id,
                        'title' => $title,
                        'body' => $message,
                        'dateandtime' => $updated_at,
                        'cp_push' => '0',
                        'message_section' => '0',
                        'read_message' => '1',
                        'dateonly' => $dateonly
                    ]);
                    //Local Notification For Mobile End


                    if (count($getproductcount) == count($getproductcount1)) {
                        DB::table('sales_order')->where('entity_id', '=', $id)->update(['state' => $state, 'status' => $status, 'order_details_block' => $order_details_block]);
                        DB::table('sales_order_grid')->where('entity_id', '=', $id)->update(['status' => $status]);
                        //Invoice Mail Trigger Start
                        $times = time();
                        $coreconfigdata = DB::table('core_config_data')->select('value')->where('scope', '=', 'default')->where('path', '=', 'web/unsecure/base_url')->first();
                        $configdomain = isset($coreconfigdata->value) ? $coreconfigdata->value : '';
                        $curl = curl_init();
                        curl_setopt_array($curl, array(
                            CURLOPT_URL => "$configdomain/api/after_delivered_taxInvoice_send_to_customer.php?order_id=$id&nocache=$times",
                            CURLOPT_RETURNTRANSFER => true,
                            CURLOPT_ENCODING => '',
                            CURLOPT_MAXREDIRS => 10,
                            CURLOPT_TIMEOUT => 0,
                            CURLOPT_FOLLOWLOCATION => true,
                            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                            CURLOPT_CUSTOMREQUEST => 'GET',
                        ));
                        $response = curl_exec($curl);
                        curl_close($curl);
                        // echo $response;
//Invoice Mail Trigger End
                    }
                    DB::table('sales_order_status_history')->insert([
                        'parent_id' => $id,
                        'is_customer_notified' => 0,
                        'is_visible_on_front' => 0,
                        'comment' => $status,
                        'status' => $status,
                        'created_at' => $created,
                        'entity_name' => 'order'
                    ]);

                    // if($order_status_id=="5"){
// $ngenius = DB::table('ngenius_networkinternational')->where('entity_id','=',$id)->get();
// if(count($ngenius)==1){
// $amount=isset($increment[0]->grand_total) ? $increment[0]->grand_total : '';  
// $reference=isset($ngenius[0]->reference) ? $ngenius[0]->reference : ''; 
// $payment_id=isset($ngenius[0]->payment_id) ? $ngenius[0]->payment_id : ''; 
// $amount1=$amount * 100;

                    // $apikey_ref= DB::table('core_config_data')->where('path','=','payment/ngeniusonline/api_key')->get();
// $apikey=$apikey_ref[0]->value;   
// $outlet_ref= DB::table('core_config_data')->where('path','=','payment/ngeniusonline/outlet_ref')->get();
// $outlet=$outlet_ref[0]->value; // enter your API key here
// $environment_ref= DB::table('core_config_data')->where('path','=','payment/ngeniusonline/environment')->get();
// $environment=$environment_ref[0]->value;
// if($environment=="uat"){
//  $paymenturl="https://api-gateway.sandbox.ngenius-payments.com/";   
// }else{
//  $paymenturl="https://api-gateway.ngenius-payments.com/";   
// }
// $ch = curl_init(); 
// curl_setopt($ch,CURLOPT_URL,$paymenturl."identity/auth/access-token"); 
// curl_setopt($ch,CURLOPT_HTTPHEADER,array(
//     "accept: application/vnd.ni-identity.v1+json",
//     "authorization: Basic ".$apikey,
//     "content-type: application/vnd.ni-identity.v1+json"
//   )); 
// curl_setopt($ch,CURLOPT_RETURNTRANSFER,true);   
// curl_setopt($ch,CURLOPT_POST, 1); 
// if($environment=="uat"){
// curl_setopt($ch, CURLOPT_POSTFIELDS,  "{\"realmName\":\"ni\"}"); 
// }else{
// curl_setopt($ch, CURLOPT_POSTFIELDS,  "{\"realmName\":\"networkinternational\"}");    
// }
// $output = json_decode(curl_exec($ch)); 
// $access_token = $output->access_token;

                    // $token=$access_token; 

                    // $ch = curl_init(); 
// $json=array("amount"=>array("currencyCode"=>"AED","value"=>"$amount1"));
// curl_setopt($ch, CURLOPT_URL,$paymenturl."transactions/outlets/".$outlet."/orders/".$reference."/payments/".$payment_id."/captures"); 
// curl_setopt($ch, CURLOPT_HTTPHEADER, array(
//     "Authorization: Bearer ".$token, 
//     "Content-Type: application/vnd.ni-payment.v2+json", 
//     "Accept: application/vnd.ni-payment.v2+json")); 
// curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);   
// curl_setopt($ch, CURLOPT_POST, 1); 
// curl_setopt($ch, CURLOPT_POSTFIELDS,json_encode($json)); 

                    // $output = json_decode(curl_exec($ch));   
// DB::table('ngenius_networkinternational')->where('entity_id','=',$id)->update(['status'=>"ngenius_complete",'state'=>"CAPTURED",'captured_amt'=>$amount]);   
// curl_close ($ch);    
// }
// }

                }

                // $order_id=$id;
                // $salesinvoice=$connection->fetchAll("SELECT sales_invoice.order_id FROM sales_invoice INNER JOIN sales_order ON sales_order.entity_id=sales_invoice.order_id WHERE sales_invoice.order_id='$order_id'");   
                // if(count($salesinvoice)==0){
                // $InvoiceService = $objectManager->create('\Magento\Sales\Model\Service\InvoiceService');
                // $transaction = $objectManager->create('\Magento\Framework\DB\Transaction');
                // $loadorder = $objectManager->create('\Magento\Sales\Api\Data\OrderInterfaceFactory')->create();
                // $salesorder=$connection->fetchAll("SELECT increment_id FROM sales_order WHERE entity_id='$order_id'");   
                // $increment_id=isset($salesorder[0]['increment_id']) ? $salesorder[0]['increment_id'] : '';
                // $order = $loadorder->loadByIncrementId($increment_id);
                // $invoice = $InvoiceService->prepareInvoice($order);
                // $invoice->register();
                // $invoice->save();
                // $transactionSave = $transaction->addObject($invoice)->addObject($invoice->getOrder());
                // $invoice->getOrder()->setIsInProcess(true);
                // $transactionSave->save();
                // $order->addStatusHistoryComment(__('Invoice has been created #%1.', $invoice->getId()))->setIsCustomerNotified(true)->save();
                // $invoiceSender = $objectManager->get('Magento\Sales\Model\Order\Email\Sender\InvoiceSender');
                // $invoiceSender->send($invoice);
                // }


                // if (isset($input['order_status_id']) && $input['order_status_id'] == 5 && !empty($order)) {
                //     $this->paymentRepository->update(['status' => 'Paid'], $order['payment_id']);
                // }
                // event(new OrderChangedEvent($oldStatus, $order));

                // if (setting('enable_notifications', false)) {
                //     if (isset($input['order_status_id']) && $input['order_status_id'] != $oldOrder->order_status_id) {
                //         Notification::send([$order->user], new StatusChangedOrder($order));
                //     }

                //     if (isset($input['driver_id']) && ($input['driver_id'] != $oldOrder['driver_id'])) {
                //         $driver = $this->userRepository->findWithoutFail($input['driver_id']);
                //         if (!empty($driver)) {
                //             Notification::send([$driver], new AssignedOrder($order));
                //         }
                //     }
                // }
            }
        } catch (ValidatorException $e) {
            return $this->sendError($e->getMessage());
        }
        if ($not_delivered == 0) {
            $checkaction = DB::table('sales_order_item')->select('order_id')->where('order_id', '=', $id)->where('actionrequired', '=', '1')->get();
            if ((count($checkaction) > 0) && $order_status_id == "3") {
                return $this->sendResponse($order->toArray(), 'Action Required.');
            } else {
                return $this->sendResponse($order->toArray(), __('lang.saved_successfully', ['operator' => __('lang.order')]));
            }
        } else {
            return response()->json([
                'success' => false, // true or false
                'data' => "",
                'message' => "Sorry, payment not captured. Please contact administrator.",
            ], 200); // 200, 401, 403, 404, 409, etc

            //  return $this->sendResponse("",'Sorry, Payment not captured. Please contact administrator.');          
        }
    }

}
