<?php
namespace App\Http\Controllers;

use Flash;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Response;
use Prettus\Validator\Exceptions\ValidatorException;
use DB;

class CategoriesController extends Controller
{
    public function index()
    {
         $mobile_category = DB::table('mobile_category')->orderBy('id','DESC')->paginate(10);
         return view('category/index', ['mobile_category' => $mobile_category
      ]);
    }

    /**
     * Show the form for creating a new User.
     *
     * @return Response
     */
    public function create()
    {
        $categories = DB::table('catalog_category_entity')->join('catalog_category_entity_varchar','catalog_category_entity_varchar.entity_id', '=', 'catalog_category_entity.entity_id')->select('catalog_category_entity.entity_id as id', 'catalog_category_entity_varchar.value as name')->where('catalog_category_entity_varchar.attribute_id','=','45')->where('catalog_category_entity.entity_id','!=',2)->where('catalog_category_entity.entity_id','!=',1)->where('catalog_category_entity_varchar.store_id','=',0)->get();
        return view('category.create',['categories'=>$categories]);
        
    }



    /**
     * Store a newly created User in storage.
     *
     * @param CreateUserRequest $request
     *
     * @return Response
     */
    public function store(Request $request)
    {
        if (env('APP_DEMO', false)) {
            Flash::warning('This is only demo app you can\'t change this section ');
            return redirect(route('users.index'));
        }
        try {
        $date_added	= date('Y-m-d H:i:s');
        $category=implode(',',$request->category_id);
      DB::table('mobile_category')->insert(['title'=>$request->title,'title_ar'=>$request->title_ar,'category_id'=>$category,'status'=>$request->status,'created'=>$date_added]);
        } catch (ValidatorException $e) {
            Flash::error($e->getMessage());
        }

        Flash::success('saved successfully.');

        return redirect(route('category.index'));
    }

    /**
     * Display the specified User.
     *
     * @param int $id
     *
     * @return Response
     */
    public function show($id)
    {
        $user = $this->VehicleRepository->findWithoutFail($id);

        if (empty($user)) {
            Flash::error('User not found');

            return redirect(route('category.index'));
        }

        return view('settings.users.profile')->with('user', $user);
    }

    public function loginAsUser(Request $request, $id)
    {
        $user = $this->VehicleRepository->findWithoutFail($id);
        if (empty($user)) {
            Flash::error('User not found');
            return redirect(route('users.index'));
        }
        auth()->login($user, true);
        if (auth()->id() !== $user->id) {
            Flash::error('User not found');
        }
        return redirect(route('users.profile'));
    }

    /**
     * Show the form for editing the specified User.
     *
     * @param int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        if (!auth()->user()->hasRole('admin') && $id != auth()->id()) {
            Flash::error('Permission denied');
            return redirect(route('labels.index'));
        }
       
	    $result = array();		
		$result['message'] = array();
		
		$editlabel = DB::table('mobile_category')->where('id','=',$id)->get();
		$result['editlabel'] = $editlabel;		
		$categories = DB::table('catalog_category_entity')->join('catalog_category_entity_varchar','catalog_category_entity_varchar.entity_id', '=', 'catalog_category_entity.entity_id')->select('catalog_category_entity.entity_id as id', 'catalog_category_entity_varchar.value as name')->where('catalog_category_entity_varchar.attribute_id','=','45')->where('catalog_category_entity.entity_id','!=',2)->where('catalog_category_entity.entity_id','!=',1)->where('catalog_category_entity_varchar.store_id','=',0)->get();
		$result['categories'] = $categories;
        if (empty($editlabel)) {
            Flash::error('category not found');

            return redirect(route('category.index'));
        }
        
        return view("category.edit",['editlabel' => $editlabel])->with('result',$result);
        
    }

    /**
     * Update the specified User in storage.
     *
     * @param int $id
     * @param UpdateUserRequest $request
     *
     * @return Response
     */
    public function update($id,Request $request)
    {
        if (env('APP_DEMO', false)) {
            Flash::warning('This is only demo app you can\'t change this section ');
            return redirect(route('users.profile'));
        }
        if (!auth()->user()->hasRole('admin') && $id != auth()->id()) {
            Flash::error('Permission denied');
            return redirect(route('users.profile'));
        }

        $lang_translation = DB::table('mobile_category')->where('id','=',$request->id)->get();
        if (empty($lang_translation)) {
            Flash::error('Label not found');
            return redirect(route('users.profile'));
        }
        try {
        $last_modified=date('Y-m-d H:i:s');
		DB::table('mobile_category')->where('id','=',$id)->update(['title'=>$request->title,'title_ar'=>$request->title_ar,'category_id'=>implode(',',$request->category_id),'status'=>$request->status,'modified'=>$last_modified]);
        } catch (ValidatorException $e) {
            Flash::error($e->getMessage());
        }


        Flash::success('category updated successfully.');

        return redirect()->back();

    }

    /**
     * Remove the specified User from storage.
     *
     * @param int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        if (env('APP_DEMO', false)) {
            Flash::warning('This is only demo app you can\'t change this section ');
            return redirect(route('users.index'));
        }
        
        $lang_translation=DB::table('mobile_category')->where('id','=',$id)->get();
		
        if (empty($lang_translation)) {
            Flash::error('category not found');

            return redirect(route('category.index'));
        }

        DB::table('mobile_category')->where('id','=',$id)->delete();

        Flash::success('category deleted successfully.');

        return redirect(route('category.index'));
    }

    /**
     * Remove Media of User
     * @param Request $request
     */
    public function removeMedia(Request $request)
    {
        if (env('APP_DEMO', false)) {
            Flash::warning('This is only demo app you can\'t change this section ');
        } else {
            if (auth()->user()->can('medias.delete')) {
                $input = $request->all();
                $user = $this->VehicleRepository->findWithoutFail($input['id']);
                try {
                    if ($user->hasMedia($input['collection'])) {
                        $user->getFirstMedia($input['collection'])->delete();
                    }
                } catch (\Exception $e) {
                    Log::error($e->getMessage());
                }
            }
        }
    }
}
