<?php

namespace App\Http\Controllers;

use App\DataTables\CategoryDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateCategoryRequest;
use App\Http\Requests\UpdateCategoryRequest;
use App\Repositories\CategoryRepository;
use App\Repositories\CustomFieldRepository;
use App\Repositories\UploadRepository;
use Flash;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Response;
use Prettus\Validator\Exceptions\ValidatorException;
use DB;

class SectionController extends Controller
{
    /** @var  CategoryRepository */
    private $categoryRepository;

    /**
     * @var CustomFieldRepository
     */
    private $customFieldRepository;

    /**
  * @var UploadRepository
  */
private $uploadRepository;

    public function __construct(CategoryRepository $categoryRepo, CustomFieldRepository $customFieldRepo , UploadRepository $uploadRepo)
    {
        parent::__construct();
        $this->categoryRepository = $categoryRepo;
        $this->customFieldRepository = $customFieldRepo;
        $this->uploadRepository = $uploadRepo;
    }

    /**
     * Display a listing of the Category.
     *
     * @param CategoryDataTable $categoryDataTable
     * @return Response
     */
    public function index(CategoryDataTable $categoryDataTable)
    {
        $mobilesection = DB::table('mobile_section')->groupby('section_name')->distinct()->orderby('section_name','asc')->get();
       
         return view('mobile_section/index', ['mobilesection' => $mobilesection
      ]);
    }

    /**
     * Show the form for creating a new Category.
     *
     * @return Response
     */
    public function create(Request $request)
    {
        
   // $edit=DB::table('mobile_section')->orderBy('id', 'desc')->first();
    	  
        //  DB::table('mobile_section')->insert(['section_name'=>$request->title,'section_id'=>$edit->section_id,'category_id'=>""]);
        //      Flash::success(__('lang.saved_successfully',['operator' => __('lang.category')]));
        	
      return view('mobile_section.create');
        //   return redirect(route('mobile_section.index'));
    }
    public function sectioncreate(Request $request)
    {
    $edit=DB::table('mobile_section')->where('section_name', $request->title)->first();
    	  if($edit){
        
             Flash::success(__('section Already Found',['operator' => __('lang.category')]));
    	  }else{
    	       DB::table('mobile_section')->insert(['section_name'=>$request->title,'section_id'=>'1']);
             Flash::success(__('lang.saved_successfully',['operator' => __('lang.category')]));
    	  }
     
          return redirect(route('mobile_section.index'));
    }
    /**
     * Store a newly created Category in storage.
     *
     * @param CreateCategoryRequest $request
     *
     * @return Response
     */
    public function store(CreateCategoryRequest $request)
    {
        $input = $request->all();
        $customFields = $this->customFieldRepository->findByField('custom_field_model', $this->categoryRepository->model());
        try {
            $category = $this->categoryRepository->create($input);
            $category->customFieldsValues()->createMany(getCustomFieldsValues($customFields,$request));
            if(isset($input['image']) && $input['image']){
    $cacheUpload = $this->uploadRepository->getByUuid($input['image']);
    $mediaItem = $cacheUpload->getMedia('image')->first();
    $mediaItem->copy($category, 'image');
}
        } catch (ValidatorException $e) {
            Flash::error($e->getMessage());
        }

        Flash::success(__('lang.saved_successfully',['operator' => __('lang.category')]));

        return redirect(route('categories.index'));
    }

    /**
     * Display the specified Category.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        $category = $this->categoryRepository->findWithoutFail($id);

        if (empty($category)) {
            Flash::error('Category not found');

            return redirect(route('categories.index'));
        }

        return view('categories.show')->with('category', $category);
    }

    /**
     * Show the form for editing the specified Category.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        $category = $this->categoryRepository->findWithoutFail($id);
        
           $section=DB::table('mobile_section')->where('id', $id)->first();

         $sectiond=DB::table('mobile_section')->where('section_name', $section->section_name)->get();

        return view('mobile_section.edit')->with('section', $sectiond)->with('id',$id);
    }
    public function editssection($id)
    {
        $categories = DB::table('catalog_category_entity')->join('catalog_category_entity_varchar','catalog_category_entity_varchar.entity_id', '=', 'catalog_category_entity.entity_id')->select('catalog_category_entity.entity_id as id', 'catalog_category_entity_varchar.value as name')->where('catalog_category_entity_varchar.attribute_id','=','45')->where('catalog_category_entity.entity_id','!=',2)->where('catalog_category_entity.entity_id','!=',1)->where('catalog_category_entity_varchar.store_id','=',0)->get();
        $category = $this->categoryRepository->findWithoutFail($id);
        
           $section=DB::table('mobile_section')->where('id', $id)->get();

        return view('mobile_section.sectionedit')->with('section', $section)->with('categories',$categories);
    }
       public function sectionadd($id)
    {
        $category = $this->categoryRepository->findWithoutFail($id);
        
           $section=DB::table('mobile_section')->where('section_id', $id)->get();

        

        return view('mobile_section.sectionadd')->with('section', $section);
    }
       public function sectionstore(Request $request,$id)
    {
        
       $get=DB::table('mobile_section')->where('section_name','=',$request->title)->where('category_id','=',$request->cat_id)->first();
       if($get){
        if($request->file('image')){
         $imageName = $request->file('image')->getClientOriginalName();
        
         $request->image->move(public_path('images'), $imageName);
 
   
            $imagepath=public_path('images')."/".$imageName;
        
         
        
      	DB::table('mobile_section')->where('id','=',$id)->update(['category_id'=>$request->cat_id,'image_url'=>$imageName,'screens'=>$request->screens]);
             Flash::success(__('lang.saved_successfully',['operator' => __('lang.category')]));
        	}else{
        	    DB::table('mobile_section')->where('id','=',$id)->update(['category_id'=>$request->cat_id,'screens'=>$request->screens]);
        	     Flash::success(__('lang.saved_successfully',['operator' => __('lang.category')])); 
        	}
       }else{
           
           $get1=DB::table('mobile_section')->where('section_name',$request->title)->orderby('id','desc')->first();
           if($get1->category_id=="0"){
                 $section_id=$get1->section_id + 1;
                     if($request->file('image')){
             $imageName = $request->file('image')->getClientOriginalName();
        
         $request->image->move(public_path('images'), $imageName);
 
   
            $imagepath=public_path('images')."/".$imageName;
            }else{
                $imageName="";
            }
                 DB::table('mobile_section')->where('id','=',$id)->update(['category_id'=>$request->cat_id,'section_id'=>$section_id,'image_url'=>$imageName,'screens'=>$request->screens,'section_name'=>$request->title]);
           }
           else{
           $section_id=$get1->section_id + 1;
            if($request->file('image')){
             $imageName = $request->file('image')->getClientOriginalName();
        
         $request->image->move(public_path('images'), $imageName);
 
   
            $imagepath=public_path('images')."/".$imageName;
            }else{
                $imageName="";
            }
         DB::table('mobile_section')->insert(['category_id'=>$request->cat_id,'section_id'=>$section_id,'screens'=>$request->screens,'image_url'=>$imageName,'section_name'=>$request->title]);
      
         
           }
       }
          
          return redirect(route('mobile_section.index'));
    }
    /**
     * Update the specified Category in storage.
     *
     * @param  int              $id
     * @param UpdateCategoryRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateCategoryRequest $request)
    {
        $category = $this->categoryRepository->findWithoutFail($id);

        if (empty($category)) {
            Flash::error('Category not found');
            return redirect(route('categories.index'));
        }
        $input = $request->all();
        $customFields = $this->customFieldRepository->findByField('custom_field_model', $this->categoryRepository->model());
        try {
            $category = $this->categoryRepository->update($input, $id);
            
            if(isset($input['image']) && $input['image']){
    $cacheUpload = $this->uploadRepository->getByUuid($input['image']);
    $mediaItem = $cacheUpload->getMedia('image')->first();
    $mediaItem->copy($category, 'image');
}
            foreach (getCustomFieldsValues($customFields, $request) as $value){
                $category->customFieldsValues()
                    ->updateOrCreate(['custom_field_id'=>$value['custom_field_id']],$value);
            }
        } catch (ValidatorException $e) {
            Flash::error($e->getMessage());
        }

        Flash::success(__('lang.updated_successfully',['operator' => __('lang.category')]));

        return redirect(route('categories.index'));
    }

    /**
     * Remove the specified Category from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        $get=DB::table('mobile_section')->where('id','=',$id)->first();
        DB::table('mobile_section')->where('section_name','=',$get->section_name)->delete();
           Flash::success(__('Deleted Successfully',['operator' => __('lang.category')]));
        return redirect(route('mobile_section.index'));
    }
    public function sectiondestroy($id)
    {
      
        DB::table('mobile_section')->where('id','=',$id)->delete();
           Flash::success(__('Deleted Successfully',['operator' => __('lang.category')]));
        return redirect(route('mobile_section.index'));
    }
        /**
     * Remove Media of Category
     * @param Request $request
     */
    public function removeMedia(Request $request)
    {
        $input = $request->all();
        $category = $this->categoryRepository->findWithoutFail($input['id']);
        try {
            if($category->hasMedia($input['collection'])){
                $category->getFirstMedia($input['collection'])->delete();
            }
        } catch (\Exception $e) {
            Log::error($e->getMessage());
        }
    }
}
